package com.jiuqi.util.inifile;

import com.jiuqi.util.Stream;
import java.util.ArrayList;
import java.util.List;

public class Section {
   private String name;
   private List list;
   private IniBuffer buffer;
   private boolean founded;

   public Section(String aName, IniBuffer aBuffer) {
      this.name = aName;
      this.buffer = aBuffer;
      this.list = new ArrayList();
   }

   public String name() {
      return this.name;
   }

   public int count() {
      return this.list.size();
   }

   public Ident get(int index) {
      return (Ident)this.list.get(index);
   }

   public void readInfo() {
      this.clear();
      int l = this.buffer.readInfoInteger();

      for(int i = 0; i < l; ++i) {
         Ident node = new Ident();
         node.name = this.buffer.readInfoString();
         node.type = this.buffer.readInfoByte();
         node.position = this.buffer.readInfoInteger();
         this.list.add(node);
         if (node.type == 7) {
            node.child = new BufferIni(this.buffer);
            node.child.readInfo();
         }
      }

   }

   public void writeInfo() {
      int l = this.count();
      this.buffer.writeInfoInteger(l);

      for(int i = 0; i < l; ++i) {
         Ident node = this.get(i);
         this.buffer.writeInfoString(node.name);
         this.buffer.writeInfoByte(node.type);
         this.buffer.writeInfoInteger(node.position);
         if (node.child != null) {
            node.child.writeInfo();
         }
      }

   }

   public void clear() {
      this.list.clear();
   }

   public void deleteKey(int index) {
      this.list.remove(index);
   }

   public void deleteKey(String ident) {
      int i = this.find(ident);
      if (this.founded) {
         this.list.remove(i);
      }

   }

   public void writeNull(String ident) {
      this.deleteKey(ident);
   }

   public void writeBoolean(String ident, boolean value) {
      Ident node = this.findNodeCreate(ident);
      node.type = 1;
      node.child = null;
      node.position = this.buffer.writeBoolean(value);
   }

   public void writeDouble(String ident, double value) {
      Ident node = this.findNodeCreate(ident);
      node.type = 3;
      node.child = null;
      node.position = this.buffer.writeDouble(value);
   }

   public void writeDateTime(String ident, long value) {
      this.writeLong(ident, value);
   }

   public void writeInteger(String ident, int value) {
      Ident node = this.findNodeCreate(ident);
      node.type = 2;
      node.child = null;
      node.position = this.buffer.writeInteger(value);
   }

   public void writeLong(String ident, long value) {
      Ident node = this.findNodeCreate(ident);
      node.type = 4;
      node.child = null;
      node.position = this.buffer.writeLong(value);
   }

   public void writeString(String ident, String value) {
      if (value == null) {
         this.deleteKey(ident);
      } else {
         Ident node = this.findNodeCreate(ident);
         node.type = 5;
         node.child = null;
         node.position = this.buffer.writeString(value);
      }

   }

   public void writeStream(String ident, Stream stream) {
      try {
         Ident node = this.findNodeCreate(ident);
         node.type = 6;
         node.child = null;
         int l = (int)(stream.getSize() - stream.getPosition());
         node.position = this.buffer.writeInteger(l);
         this.buffer.writeStream(stream, l);
      } catch (Exception var5) {
         Exception ex = var5;
         ex.printStackTrace();
      }

   }

   public BufferIni writeIni(String ident) {
      Ident node = this.findNodeCreate(ident);
      node.type = 7;
      if (node.child == null) {
         node.child = new BufferIni(this.buffer);
      }

      return node.child;
   }

   public int find(String ident) {
      this.founded = false;
      int l = 0;
      int h = this.list.size() - 1;

      while(l <= h) {
         int i = (l + h) / 2;
         int c = this.get(i).name.compareTo(ident);
         if (c < 0) {
            l = i + 1;
         } else {
            h = i - 1;
            if (c == 0) {
               this.founded = true;
               l = i;
            }
         }
      }

      return l;
   }

   public Ident findNode(String ident) {
      int i = this.find(ident);
      return this.founded ? this.get(i) : null;
   }

   public Ident findNodeCreate(String ident) {
      int i = this.find(ident);
      if (this.founded) {
         return this.get(i);
      } else {
         Ident ret = new Ident();
         ret.name = ident;
         this.list.add(i, ret);
         return ret;
      }
   }

   public boolean isNull(String ident) {
      Ident node = this.findNode(ident);
      return node == null;
   }

   public BufferIni readIni(String ident) {
      int i = this.find(ident);
      return this.founded ? this.get(i).child : null;
   }

   public boolean readBoolean(String ident, boolean defValue) {
      Ident node = this.findNode(ident);
      return node != null ? this.buffer.readBoolean(node.position) : defValue;
   }

   public double readDouble(String ident, double defValue) {
      Ident node = this.findNode(ident);
      return node != null ? this.buffer.readDouble(node.position) : defValue;
   }

   public long readDateTime(String ident, long defValue) {
      return this.readLong(ident, defValue);
   }

   public int readInteger(String ident, int defValue) {
      Ident node = this.findNode(ident);
      return node != null ? this.buffer.readInteger(node.position) : defValue;
   }

   public long readLong(String ident, long defValue) {
      Ident node = this.findNode(ident);
      return node != null ? this.buffer.readLong(node.position) : defValue;
   }

   public String readString(String ident, String defValue) {
      Ident node = this.findNode(ident);
      return node != null ? this.buffer.readString(node.position) : defValue;
   }

   public void readStream(String ident, Stream stream) {
      Ident node = this.findNode(ident);
      if (node != null) {
         try {
            int l = this.buffer.readInteger(node.position);
            this.buffer.readStream(stream, node.position + 4, l);
         } catch (Exception var5) {
            Exception ex = var5;
            ex.printStackTrace();
         }
      }

   }
}
